import unittest
import wcxf


class TestWET(unittest.TestCase):
    def test_bern(self):
        basis = wcxf.Basis['WET', 'Bern']
        # comparing no. of WCs to table 1 of arXiv:1704.06639
        self.assertEqual(len(basis.sectors['sbsb']), 8)
        self.assertEqual(len(basis.sectors['dbdb']), 8)
        self.assertEqual(len(basis.sectors['ubenu']), 5*3)
        self.assertEqual(len(basis.sectors['ubmunu']), 5*3)
        self.assertEqual(len(basis.sectors['ubtaunu']), 5*3)
        self.assertEqual(len(basis.sectors['cbenu']), 5*3)
        self.assertEqual(len(basis.sectors['cbmunu']), 5*3)
        self.assertEqual(len(basis.sectors['cbtaunu']), 5*3)
        self.assertEqual(len(basis.sectors['sbuc']), 20)
        self.assertEqual(len(basis.sectors['sbcu']), 20)
        self.assertEqual(len(basis.sectors['dbuc']), 20)
        self.assertEqual(len(basis.sectors['dbcu']), 20)
        self.assertEqual(len(basis.sectors['sbsd']), 10)
        self.assertEqual(len(basis.sectors['dbds']), 10)
        self.assertEqual(len(basis.sectors['sb']), 57*2)
        self.assertEqual(len(basis.sectors['sbnunu']), 2*9)
        self.assertEqual(len(basis.sectors['db']), 57*2)
        self.assertEqual(len(basis.sectors['dbnunu']), 2*9)
        for l1 in ['e', 'mu', 'tau']:
            for l2 in ['e', 'mu', 'tau']:
                if l1 != l2:
                    self.assertEqual(len(basis.sectors['sb'+l1+l2]), 10)
                    self.assertEqual(len(basis.sectors['db'+l1+l2]), 10)

    def test_jms(self):
        basis = wcxf.Basis['WET', 'JMS']
        all_wc = {k: v for sk, sv in basis.sectors.items() for k, v in sv.items()}
        def isreal(wc):
            if 'real' in all_wc[wc] and all_wc[wc]['real']:
                return True
            else:
                return False
        def assert_len(s, n):
            L = sum([1 if isreal(k) else 2 for k in all_wc if s in k])
            try:
                self.assertEqual(L, n,
                             msg="Failed for {}".format(s))
            except Exception as exc:
                print(exc)
        # compare individual counts to tables 11-17 in arXiv:1709.04486
        assert_len('egamma_', 9*2)
        assert_len('ugamma_', 4*2)
        assert_len('dgamma_', 9*2)
        assert_len('uG_', 4*2)
        assert_len('dG_', 9*2)
        assert_len('VnunuLL_', 36)
        assert_len('VeeLL_', 36)
        assert_len('VnueLL_', 81)
        assert_len('VnuuLL_', 36)
        assert_len('VnudLL_', 81)
        assert_len('VeuLL_', 36)
        assert_len('VedLL_', 81)
        assert_len('VnueduLL_', 2*54)
        assert_len('VuuLL_', 10)
        assert_len('VddLL_', 45)
        assert_len('V1udLL_', 36)
        assert_len('V8udLL_', 36)
        assert_len('VeeRR_', 36)
        assert_len('VeuRR_', 36)
        assert_len('VedRR_', 81)
        assert_len('VuuRR_', 10)
        assert_len('VddRR_', 45)
        assert_len('V1udRR_', 36)
        assert_len('V8udRR_', 36)
        assert_len('VnueLR_', 81)
        assert_len('VeeLR_', 81)
        assert_len('VnuuLR_', 36)
        assert_len('VnudLR_', 81)
        assert_len('VeuLR_', 36)
        assert_len('VedLR_', 81)
        assert_len('VueLR_', 36)
        assert_len('VdeLR_', 81)
        assert_len('VnueduLR_', 2*54)
        assert_len('V1uuLR_', 16)
        assert_len('V8uuLR_', 16)
        assert_len('V1udLR_', 36)
        assert_len('V8udLR_', 36)
        assert_len('V1duLR_', 36)
        assert_len('V8duLR_', 36)
        assert_len('V1ddLR_', 81)
        assert_len('V8ddLR_', 81)
        assert_len('V1udduLR_', 2*36)
        assert_len('V8udduLR_', 2*36)
        assert_len('SeuRL_', 2*36)
        assert_len('SedRL_', 2*81)
        assert_len('SnueduRL_', 2*54)
        assert_len('SeeRR', 2*45)
        assert_len('SeuRR', 2*36)
        assert_len('TeuRR', 2*36)
        assert_len('SedRR', 2*81)
        assert_len('TedRR', 2*81)
        assert_len('SnueduRR', 2*54)
        assert_len('TnueduRR', 2*54)
        assert_len('S1uuRR', 2*10)
        assert_len('S8uuRR', 2*10)
        assert_len('S1udRR', 2*36)
        assert_len('S8udRR', 2*36)
        assert_len('S1ddRR', 2*45)
        assert_len('S8ddRR', 2*45)
        assert_len('S1udduRR', 2*36)
        assert_len('S8udduRR', 2*36)
        # compare total count to table 22
        ntot = sum([1 if isreal(k) else 2 for k in all_wc])
        self.assertEqual(ntot, 1+87+186+76+21+66+76+90+252+266+171+45+342+254+1+66+156+51+15+51+51+72+207+216+171+45+342+254+9+9+26+26)



import unittest
import wcxf

class TestBases(unittest.TestCase):
    def test_bases(self):
        for basis in wcxf.Basis.instances.values():
            try:
                basis.validate()
            except:
                self.fail("Basis {}-{} failed to validate".format(basis.eft, basis.basis))



import unittest
import numpy as np
import numpy.testing as npt
import yaml
import json
import pkgutil
import wcxf
from wcxf import translators

class TestBasis(unittest.TestCase):
    def test_eft(self):
        f = pkgutil.get_data('wcxf', 'data/test.eft.yml')
        eft = wcxf.EFT.load(f.decode('utf-8'))
        self.assertEqual(eft, wcxf.EFT['MyEFT'])
        self.assertEqual(eft.eft, 'MyEFT')
        self.assertIsInstance(eft.sectors, dict)
        f = pkgutil.get_data('wcxf', 'data/test.basis1.yml')
        basis = wcxf.Basis.load(f.decode('utf-8'))
        self.assertEqual(eft.known_bases, ('MyBasis 1',))
        self.assertIsInstance(eft.dump(fmt='json'), str)
        self.assertIsInstance(eft.dump(fmt='yaml'), str)

    def test_basis(self):
        f = pkgutil.get_data('wcxf', 'data/test.eft.yml')
        eft = wcxf.EFT.load(f.decode('utf-8'))
        f = pkgutil.get_data('wcxf', 'data/test.basis1.yml')
        basis = wcxf.Basis.load(f.decode('utf-8'))
        self.assertEqual(basis, wcxf.Basis['MyEFT', 'MyBasis 1'])
        self.assertEqual(basis._name, ('MyEFT', 'MyBasis 1'))
        self.assertEqual(basis.basis, 'MyBasis 1')
        basis.validate()

    def test_wc(self):
        f = pkgutil.get_data('wcxf', 'data/test.eft.yml')
        eft = wcxf.EFT.load(f.decode('utf-8'))
        f = pkgutil.get_data('wcxf', 'data/test.basis1.yml')
        basis = wcxf.Basis.load(f.decode('utf-8'))
        f = pkgutil.get_data('wcxf', 'data/test.wcs.yml')
        wc = wcxf.WC.load(f.decode('utf-8'))
        self.assertEqual(wc.eft, 'MyEFT')
        self.assertEqual(wc.basis, 'MyBasis 1')
        wc.validate()
        self.assertEqual(wc.scale, 1e16)
        self.assertEqual(wc.dict['C_1'], 0.12)
        self.assertEqual(wc.dict['C_2'], 0.3156-0.53j)

    def test_translator(self):
        # A trivial translator translating from MyBasis 1 to MyBasis 2
        @wcxf.translator('MyEFT', 'MyBasis 1', 'MyBasis 2')
        def f(x, parameters):
            return x
        self.assertIn(('MyEFT', 'MyBasis 1', 'MyBasis 2'), wcxf.Translator.instances)
        f = pkgutil.get_data('wcxf', 'data/test.wcs.yml')
        wc = wcxf.WC.load(f.decode('utf-8'))
        wc_out = wcxf.Translator['MyEFT', 'MyBasis 1', 'MyBasis 2'].translate(wc)
        self.assertDictEqual(
            {k: v for k, v in wc.__dict__.items() if k[0] != '_' and k != 'basis'},
            {k: v for k, v in wc_out.__dict__.items() if k[0] != '_' and k != 'basis'}
        )
        self.assertIn(('MyEFT', 'MyBasis 1', 'MyBasis 2'),
                      wcxf.Basis['MyEFT', 'MyBasis 1'].known_translators['from'])
        self.assertIn(('MyEFT', 'MyBasis 1', 'MyBasis 2'),
                      wcxf.Basis['MyEFT', 'MyBasis 2'].known_translators['to'])
        self.assertIn(('MyEFT', 'MyBasis 1', 'MyBasis 2'),
                      wcxf.EFT['MyEFT'].known_translators)
        # remove dummy translator
        del wcxf.Translator['MyEFT', 'MyBasis 1', 'MyBasis 2']

    def test_matcher(self):
        # A trivial translator translating from MyBasis 1 to MyBasis 2
        @wcxf.matcher('MyEFT', 'MyBasis 1', 'MyOtherEFT', 'MyOtherBasis 1')
        def f(x, parameters):
            return x
        # self.assertIn(('MyEFT', 'MyBasis 1', 'MyBasis 2'), wcxf.Translator.instances)
        f = pkgutil.get_data('wcxf', 'data/test.wcs.yml')
        wc = wcxf.WC.load(f.decode('utf-8'))
        wc_out = wcxf.Matcher['MyEFT', 'MyBasis 1', 'MyOtherEFT', 'MyOtherBasis 1'].match(wc)
        # self.assertIn(('MyEFT', 'MyBasis 1', 'MyBasis 2'),
        #               wcxf.Basis['MyEFT', 'MyBasis 1'].known_translators['from'])
        # self.assertIn(('MyEFT', 'MyBasis 1', 'MyBasis 2'),
        #               wcxf.Basis['MyEFT', 'MyBasis 2'].known_translators['to'])
        # self.assertIn(('MyEFT', 'MyBasis 1', 'MyBasis 2'),
        #               wcxf.EFT['MyEFT'].known_translators)
        # remove dummy matcher
        del  wcxf.Matcher['MyEFT', 'MyBasis 1', 'MyOtherEFT', 'MyOtherBasis 1']

    def test_inheritance(self):
        f = pkgutil.get_data('wcxf', 'data/test.basis1.yml')
        parent = wcxf.Basis.load(f.decode('utf-8'))
        f = pkgutil.get_data('wcxf', 'data/test.basis2.yml')
        child = wcxf.Basis.load(f.decode('utf-8'))
        self.assertDictEqual(parent.sectors, child.sectors)



from .classes import *
from . import matchers
from . import translators

# read all EFTs and bases from the wcxf-bases submodule

import os
import glob

_root = os.path.abspath(os.path.dirname(__file__))
all_efts = glob.glob(os.path.join(_root, 'bases', '*.eft.json'))
all_bases = glob.glob(os.path.join(_root, 'bases', '*.basis.json'))
child_bases = glob.glob(os.path.join(_root, 'bases', 'child', '*.basis.json'))

for eft in all_efts:
    with open(eft, 'r') as f:
        EFT.load(f)

for basis in all_bases + child_bases:
    with open(basis, 'r') as f:
        Basis.load(f)



import unittest
import numpy as np
import numpy.testing as npt
import yaml
import json
import pkgutil
import wcxf

class TestSMEFT(unittest.TestCase):
    def test_smeft_number(self):
        """Test the number of B and L conserving coefficients in SMEFT"""
        basis = wcxf.Basis['SMEFT', 'Warsaw']
        sector = basis.sectors['dB=dL=0']
        count=0
        for wc in sector:
            if 'real' not in sector[wc] or not sector[wc]['real']:
                count +=2 # complex coeffs
            else:
                count +=1 # real coeffs
        self.assertEqual(count, 2499) # no. of B and L conserving coefficients



from setuptools import setup, find_packages

setup(name='wcxf',
      version='0.1',
      author='David M. Straub, Jason Aebischer',
      author_email='david.straub@tum.de, jason.aebischer@tum.de',
      license='MIT',
      packages=['wcxf'],
      package_data={
      'wcxf':['data/*.yml']
      },
      install_requires=['pyyaml'],
      entry_points={
        'console_scripts': [
            'wcxf = wcxf.cli:wcxf_cli',
            'wcxf2eos = wcxf.cli:eos',
        ]
      },
    )



import argparse
import wcxf
import sys
import logging
import os
import yaml


def wcxf_cli():
    parser = argparse.ArgumentParser(description="Command line interface to manipulate WCxf files.")
    subparsers = parser.add_subparsers(title='subcommands')

    # convert

    parser_convert = subparsers.add_parser('convert',
                                           description="Command line script to convert WCxf files between YAML and JSON.",
                                           help="convert between YAML and JSON formats")
    parser_convert.add_argument("FORMAT", type=str,
                                help="Output format (should be yaml or json)")
    parser_convert.add_argument("FILE", nargs='?', type=argparse.FileType('r'),
                                default=sys.stdin,
                                help="Input file. If \"-\", read from standard input")
    parser_convert.add_argument("--output", nargs='?',
                                      type=argparse.FileType('w'),
                                      default=sys.stdout,
                                      help="Output file. If absent, print to standard output")
    parser_convert.set_defaults(func=convert)

    # translate

    parser_translate = subparsers.add_parser('translate',
                                             description="Command line script for basis translation of WCxf files.",
                                             help="Translate between different bases")
    parser_translate.add_argument("BASIS", help="Output basis", type=str)
    parser_translate.add_argument("FILE", nargs='?',
                                  type=argparse.FileType('r'),
                                  default=sys.stdin,
                                  help="Input file. If \"-\", read from standard input")
    parser_translate.add_argument("--output", nargs='?',
                                  type=argparse.FileType('w'), default=sys.stdout,
                                  help="Output file. If absent, print to standard output")
    parser_translate.add_argument("--format", type=str,
                                  default="json",
                                  help="Output format (default: json)")
    parser_translate.set_defaults(func=translate)

    # match

    parser_match = subparsers.add_parser('match',
                                         description="Command line script for matching of WCxf files.",
                                         help="Match between different EFTs")
    parser_match.add_argument("EFT", help="Output EFT", type=str)
    parser_match.add_argument("BASIS", help="Output basis", type=str)
    parser_match.add_argument("FILE", nargs='?',
                              type=argparse.FileType('r'), default=sys.stdin,
                              help="Input file. If \"-\", read from standard input")
    parser_match.add_argument("--output", nargs='?',
                              type=argparse.FileType('w'), default=sys.stdout,
                              help="Output file. If absent, print to standard output")
    parser_match.add_argument("--format", type=str, default="json",
                              help="Output format (default: json)")
    parser_match.set_defaults(func=match)

    # validate

    parser_validate = subparsers.add_parser('validate',
                                            description="Command line script for validation of WCxf files.",
                                            help="Validate basis or Wilson coefficient files")
    parser_validate.add_argument("TYPE", type=str,
                                       help="Type of file to validate: should be 'eft', 'basis', or 'wc'")
    parser_validate.add_argument("FILE", nargs='?',
                                 type=argparse.FileType('r'), default=sys.stdin,
                                 help="Input file. If \"-\", read from standard input")
    parser_validate.set_defaults(func=validate)

    args = parser.parse_args()
    try:
        args.func(args)
    except AttributeError:
        parser.print_help()


def convert(args):
    from wcxf.converters.yamljson import convert_json, convert_yaml
    if args.FORMAT.lower() == 'json':
        convert_json(args.FILE, args.output)
    if args.FORMAT.lower() == 'yaml':
        convert_yaml(args.FILE, args.output)


def translate(args):
    wc_in = wcxf.WC.load(args.FILE)
    wc_out = wc_in.translate(args.BASIS)
    wc_out.dump(stream=args.output, fmt=args.format)


def match(args):
    wc_in = wcxf.WC.load(args.FILE)
    wc_out = wc_in.match(args.EFT, args.BASIS)
    wc_out.dump(stream=args.output, fmt=args.format)


def validate(args):
    if args.TYPE == 'eft':
        eft = wcxf.EFT.load(args.FILE)
    elif args.TYPE == 'basis':
        basis = wcxf.Basis.load(args.FILE)
        basis.validate()
    elif args.TYPE == 'wc':
        wc = wcxf.WC.load(args.FILE)
        wc.validate()
    else:
        logging.error("TYPE should be 'eft', 'basis', or 'wc'")
        return 1
    print("Validation successful.")
    return 0


def eos():
    from wcxf.converters.eos import wcxf2eos, get_sm_wcs
    parser = argparse.ArgumentParser(description="""Command line script to convert a WCxf file to an EOS Wilson coefficient parameter file.""",
                                     formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument("FILE", nargs='?', help="Input file. If \"-\", read from standard input",
                        type=argparse.FileType('r'), default=sys.stdin)
    parser.add_argument("--eosprefix", help="Installation prefix for the EOS installation. Defaults to /usr",
                        default='/usr')
    parser.add_argument("--output", nargs='?', help="Output file. If absent, print to standard output",
                        type=argparse.FileType('w'), default=sys.stdout)
    parser.add_argument("--eoshome", help="EOS home directory. If specified, values will be written to EOSHOME/parameters/wcxf.yaml. Cannot be used simultaneously with output",
                        default=None)
    args = parser.parse_args()
    # check sanity of inputs
    if args.output != sys.stdout and args.eoshome is not None:
        logging.error("Cannot use --output and --eoshome arguments simultaneously")
        return 1
    elif args.output == sys.stdout and args.eoshome is not None:
        output_dir = os.path.join(args.eoshome, 'parameters')
        if not os.path.isdir(output_dir):
            logging.error("Output directory {} does not exist".format(output_dir))
            return 1
        f = open(os.path.join(output_dir, 'wcxf.yaml'), 'w')
    else:
        f = args.output
    # read in & validate WCxf file
    wc = wcxf.WC.load(args.FILE)
    wc.validate()
    # read EOS SM contributions
    sm_wc_dict = get_sm_wcs(os.path.join(args.eosprefix, 'share/eos', 'parameters'))
    # convert to EOS parameters
    eos_dict = wcxf2eos(wc, sm_wc_dict)
    yaml.dump(eos_dict, f, default_flow_style=False)
    f.close()
    return 0
